Boomkinator = LibStub("AceAddon-3.0"):NewAddon("Boomkinator", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")

local L    = LibStub("AceLocale-3.0"):GetLocale("Boomkinator")

-- localize functions
local UnitGUID = _G.UnitGUID
local UnitExists = _G.UnitExists
local UnitBuff = _G.UnitBuff
local UnitDebuff = _G.UnitDebuff
local UnitClass = _G.UnitClass
local UnitPower = _G.UnitPower
local GetSpellInfo = _G.GetSpellInfo
local GetSpellCooldown = _G.GetSpellCooldown
local GetSpellTexture = _G.GetSpellTexture
local UnitHealth = _G.UnitHealth
local UnitHealthMax = _G.UnitHealthMax
local GetSpecialization = _G.GetSpecialization
local sformat = string.format
--
local settings
local updater = ""

local spells = {}
local icons = {}
local buttons = {}
local timers = {}
local stacks = {}

local wreak = false
local implode = false
local blaze = false
local burn = false
local slice = false
local immolateTable = {}

Boomkinator.Credits1 = "Boomkinator - by Aznamir (Lightbringer US)";

-------------------------------------------------------------------
-- Interface settings
-------------------------------------------------------------------
Boomkinator.options =
{
	type = "group",
	name = "Boomkinator",
	args = {
		desc = {
			type = "description",
			order = 0,
			name = L["MOVE_DESC"],
		},
		reset = {
			type = "execute",
			order = 1,
			name = L["RESET"],
			desc = L["RESET_DESC"],
			func = function() Boomkinator:Reset() end,
		},
		show = {
			type = "execute",
			order = 2,
			name = L["SHOW"],
			desc = L["SHOW_DESC"],
			func = function() Boomkinator:ShowLayout() end,
		},
		hide = {
			type = "execute",
			order = 3,
			name = L["HIDE"],
			desc = L["HIDE_DESC"],
			func = function() Boomkinator:HideLayout() end,
		},
		lock = {
			type = "execute",
			order = 4,
			name = L["LOCK"],
			desc = L["LOCK_DESC"],
			func = function() 
					settings.locked = true
					Boomkinator:LockLayout() 
				end,
		},
		unlock = {
			type = "execute",
			order = 5,
			name = L["UNLOCK"],
			desc = L["UNLOCK_DESC"],
			func = function()
					settings.locked = false
					Boomkinator:UnlockLayout()
				end,
		},
		scale = {
			type = "range",
			order = 6,
			name = L["BSC"],
			desc = L["BSC_DESC"],
			min = 0.4,
			max = 1.5,
			step = 0.05,
			get = function(info) return settings.buffscale end,
			set = function(info, val)
					settings.buffscale = val
					Boomkinator:UpdateLayout()
				end,
		},
		fontscale = {
			type = "range",
			order = 7,
			name = "Font Scale",
			desc = "Scale of timer fonts", 
			min = 8,
			max = 14,
			step = 1,
			get = function(info) return settings.fontscale end,
			set = function(info, val)
					settings.fontscale = val
					Boomkinator:UpdateLayout()
				end,
		},
		rows = {
			type = "range",
			order = 8,
			name = "Display Rows",
			desc = "Number of button rows", 
			min = 1,
			max = 3,
			step = 1,
			get = function(info) return settings.rows end,
			set = function(info, val)
					settings.rows = val
					Boomkinator:UpdateLayout()
				end,
		},
	},
}
-------------------------------------------------------------------
-- Default Settings
-------------------------------------------------------------------
Boomkinator.defaults = {
	profile = {
		buffscale = 0.9,
		fontscale = 10,
		locked = false,
		disabled = false,
		rows = 3
	}
}
-------------------------------------------------------------------
-- Ace Framework Events
-------------------------------------------------------------------
function Boomkinator:OnInitialize()
	--self:Print("OnInitialize")
	DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99"..Boomkinator.Credits1.."|r")
	self.db = LibStub("AceDB-3.0"):New("BoomkinatorDB", Boomkinator.defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	settings = self.db.profile
	Boomkinator.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Boomkinator", Boomkinator.options, "boom")
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Boomkinator", "Boomkinator")

	self:UpdateLayout()
end

function Boomkinator:OnProfileChanged()
	--self:Print("OnProfileChanged")
	settings = self.db.profile
	self:UpdateLayout()
end

function Boomkinator:OnEnable()
	--self:Print("OnEnable")
	settings.disabled = false
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	--self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	self:RegisterEvent("ENCOUNTER_START")
	self:RegisterEvent("ENCOUNTER_END")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:UpdateLayout()
end

function Boomkinator:OnDisable()
	--self:Print("OnDisable")
	-- events
	settings.disabled = true
	self:UpdateLayout()
end

function Boomkinator:MediaUpdate()

end
-------------------------------------------------------------------
-- Service Functions
-------------------------------------------------------------------
function Boomkinator:FormatTime(time)
	if not time or time < 0 or time == 9999 then
		return ""
	end
	local mins = floor(time / 60)
	local secs = time - (mins * 60)
	return sformat("%d:%02d", mins, secs)
end

function Boomkinator:FormatShort(time)
	if not time or time < 0 or time == 9999 then
		return ""
	end
	return sformat("%0.1f", time)
end

function Boomkinator:ReadableNumber(num, places)
    local ret
    local placeValue = ("%%.%df"):format(places or 0)
    if not num then
        return 0
    elseif num >= 1000000000000 then
        ret = placeValue:format(num / 1000000000000) .. " T" -- trillion
    elseif num >= 1000000000 then
        ret = placeValue:format(num / 1000000000) .. " B" -- billion
    elseif num >= 1000000 then
        ret = placeValue:format(num / 1000000) .. " M" -- million
    elseif num >= 1000 then
        ret = placeValue:format(num / 1000) .. "K" -- thousand
    else
        ret = num -- hundreds
    end
    return ret
end
-------------------------------------------------------------------
-- Command Prompt Response
-------------------------------------------------------------------
function Boomkinator:Reset()
	local h = _G["BoomkinatorFrame"]
	h:ClearAllPoints()
	h:SetPoint("CENTER", "UIParent", "CENTER", 0, 0)
	--self:UpdateLayout()
end

function Boomkinator:Debug()
	self:Print("Spells:")
	for i = 1, 16 do
		self:Print("  " .. i .. " = " .. (spells[i] or ""))
	end
	self:Print("Icons:")
	for i = 1, 16 do
		self:Print("  " .. i .. " = " .. (icons[i] or ""))
	end

end
-------------------------------------------------------------------
-- Internal Functions
-------------------------------------------------------------------

-------------------------------------------------------------------
-- External Event Handling
------------------------------------------------------------------
function Boomkinator:PLAYER_REGEN_ENABLED()
	--self:Print("Regen Enabled")
	self:CancelTimer(self.UpdateTimer, true)
	self:HideLayout()
end

function Boomkinator:PLAYER_REGEN_DISABLED()
	--self:Print("Regen Disabled")

	if updater == "" then
		--self:Print("Entered combat without settings, trying to fix UI")
		self:UpdateLayout()
	end
	
	if updater ~= "" then
		--self:Print(updater)
		self.UpdateTimer = self:ScheduleRepeatingTimer(updater, 0.1)
		self:ShowLayout()
	end
end

function Boomkinator:ACTIVE_TALENT_GROUP_CHANGED()
	self:UpdateLayout()
end

function Boomkinator:PLAYER_TALENT_UPDATE()
	self:UpdateLayout()
end

function Boomkinator:ENCOUNTER_START()
	immolateTable = {}
end

function Boomkinator:ENCOUNTER_END()
	immolateTable = {}
end

function Boomkinator:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local sourceGUID = select(4, ...)
	if sourceGUID == UnitGUID("player") and updater == "ButtonsDestruction" then
		local SAR = "SPELL_AURA_REMOVED"
        local SAA = "SPELL_AURA_APPLIED"
        local SARe = "SPELL_AURA_REFRESH"
        local SD = "SPELL_DAMAGE"
        local ImmolateCastID = 348
        local ImmolateDoTID = 157736
        local ConflagID = 17962
		--local destGUID = select(8, ...)
        --local subevent = select(2, ...)
        --local spellid = select(12, ...)
		
		local _, subevent, _, _, _, _, _, destGUID, _, _, _, spellid = ...
		if spellid == ImmolateDoTID and (subevent == SAA or subevent == SARe) then
			--self:Print("new immolate on " .. destGUID)
			immolateTable[destGUID] = 1
		elseif spellid == ImmolateDoTID and  subevent == SAR then
			--self:Print("deleted immolate on " .. destGUID)
			immolateTable[destGUID] = nil
		elseif spellid == ConflagID and subevent == SD then
            if immolateTable[destGUID] then
				--self:Print("increase immolate on " .. destGUID)
                immolateTable[destGUID] = immolateTable[destGUID] * 1.25
            end
		end
	end
end
-------------------------------------------------------------------
-- Internal Functions
-------------------------------------------------------------------

function Boomkinator:CheckTalents()
	local class = select(2, UnitClass("player"))
	local spec = GetSpecialization()
	local name, i
	-- druid
	if class == "DRUID" and spec == 1 then -- moonkin
		name = "Balance"
	elseif class == "DRUID" and spec == 4 then -- resto
		name = "Restoration"
	-- rogue
	elseif class == "ROGUE" and  spec == 2 then -- rogue combat
		name = "Outlaw"
	-- warlock
	elseif class == "WARLOCK" and spec == 1 then -- affliction
		name = "Affliction"
	elseif class == "WARLOCK" and spec == 2 then -- demonology
		name = "Demonology"
	elseif class == "WARLOCK" and spec == 3 then -- destruction
		name = "Destruction"
	-- demon hunter
	elseif class == "DEMONHUNTER" and spec == 1 then -- Havoc
		name = "Havoc"
	elseif class == "DEMONHUNTER" and spec == 2 then -- Vengeance
		name = "Vengeance"
	-- hunter
	elseif class == "HUNTER" and spec == 1 then -- beast mastery
		name = "BeastMastery"
	-- paladin
	elseif class == "PALADIN" and spec == 3 then -- retribution
		name = "Retribution"
	end
	if name then
		updater = "Buttons"..name
		for i = 1, 20, 1 do
			spells[i] = Boomkinator.Spells[name][i]
		end
	end
end

function Boomkinator:DoTheThing(thing, slot, ...)
-- thing, slot, parameters
-- 1, x, power
-- self buff
-- 2, x, target, spell, coloring_method(1, -1, 0), show_stacks(1, -1), stacks_max
-- debuff
-- 3, x, target, spell, coloring_method(1, -1, 0)
-- simple cooldown
-- 4, x, spell, shorttimer
-- conditional debuff on target
-- 5, x, target, spell, method, condition
-- conditional cooldown
-- 6, x, spell, condition, shorttimer
-- rechargeable spell
-- 7, x, spell, duration(1 for long)	
	local target, spell, power, powerlimit, condition, method, show_stacks, shorttimer, stacks_max
	local start, duration, expire
	local buffName, buffCount, buffDuration, buffExpire
	
	-- power check
	-- 1, x, power
	if thing == 1 then
		power = select(1, ...)
		stacks[slot]:SetText("x"..(power or "0"))
	-- self buff expiration
	-- 2, x, target, spell, method, shows tacks
	elseif thing == 2 then
		target = select (1, ...)
		spell = select(2, ...)
		method = select(3, ...)
		show_stacks = select(4, ...)
		stacks_max = select(5, ...)
		spellName = GetSpellInfo(spell)
		buffName, _, _, buffCount, _, buffDuration, buffExpire = UnitBuff(target, spellName)
		if buffName and buffExpire then
			expire = buffExpire - GetTime()
			timers[slot]:SetText(Boomkinator:FormatShort(expire))
			-- stack counts
			if show_stacks == 1 then
				if buffCount then
					stacks[slot]:SetText("x" .. buffCount)
					if stacks_max and stacks_max > 0 then
					-- color stacks
						if buffCount == stacks_max then
							stacks[slot]:SetTextColor(1, 0, 0, 1)
						else
							stacks[slot]:SetTextColor(0, 1, 0, 1)
						end
					end
				else
					stacks[slot]:SetText("")
				end
			end
			-- highlight method
			if method == 1 then
				buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
			elseif method == -1 then
				buttons[slot]:SetVertexColor(1, 1, 1, 0.25)
			end
		else
			if show_stacks == 1 then
				stacks[slot]:SetText("")
			end
			if method == 1 then 
				buttons[slot]:SetVertexColor(1, 1, 1, 0.25)
			elseif method == -1 then
				buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
			end
			timers[slot]:SetText("")
		end
		return buffCount
	 -- debuff on target
	 -- 3, x, target, spell, method
	elseif thing == 3 then
		target = select (1, ...)
		spell = select(2, ...)
		method = select(3, ...)
		if UnitExists(target) and spell then
			spellName = GetSpellInfo(spell)
			_, _, _, _, _, buffDuration, buffExpire = UnitDebuff(target, spellName, nil, "PLAYER")
			if buffExpire then
				expire = buffExpire - GetTime()
				timers[slot]:SetText(Boomkinator:FormatShort(expire))
				if method == 1 then
					buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
				elseif method == -1 then
					buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
				end
			else
				timers[slot]:SetText("")
				if method == 1 then
					buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
				elseif method == -1 then
					buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
				end
			end
		else
			timers[slot]:SetText("")
			if method == 1 then
				buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
			elseif method == -1 then
				buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
			end
		end
	-- simple cooldown
	-- 4, x, spell, shorttimer
	elseif thing == 4 then
		spell = select(1, ...)
		shorttimer = select(2, ...)
		spellName = GetSpellInfo(spell)
		start, duration = GetSpellCooldown(spellName)
		if start then
			if start == 0 or (start > 0 and duration < 1.5 ) then
				timers[slot]:SetText("")
				buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
			else
				-- on cooldown
				expire = start + duration - GetTime()
				if shorttimer == 1 then
					timers[slot]:SetText(Boomkinator:FormatShort(expire))
				else
					timers[slot]:SetText(Boomkinator:FormatTime(expire))
				end
				buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
			end
		else
			timers[slot]:SetText("NA")
			buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
		end
	-- conditional debuff on target
	-- 5, x, target, spell, method, condition
	elseif thing == 5 then
		target = select (1, ...)
		spell = select(2, ...)
		method = select(3, ...)
		condition = select(4, ...)
		if UnitExists(target) and spell then
			spellName = GetSpellInfo(spell)
			_, _, _, _, _, buffDuration, buffExpire = UnitDebuff(target, spellName, nil, "PLAYER")
			if buffExpire then
				expire = buffExpire - GetTime()
				timers[slot]:SetText(Boomkinator:FormatShort(expire))
				if method == 1 then
					buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
				elseif method == -1 then
					buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
				end
			else
				timers[slot]:SetText("")
				if condition then
					if method == 1 then
						buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
					elseif method == -1 then
						buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
					end
				else
					if method == 1 then
						buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
					elseif method == -1 then
						buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
					end
				end
			end
		else
			timers[slot]:SetText("")
			if method == 1 then
				buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
			elseif method == -1 then
				buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
			end
		end
	-- conditional cooldown
	-- 6, x, spell, condition, shorttimer
	elseif thing == 6 then 
		spell = select(1, ...)
		condition = select(2, ...)
		shorttimer = select(3, ...)
		spellName = GetSpellInfo(spell)
		start, duration = GetSpellCooldown(spellName)
		if start then
			if (start == 0 or (start > 0 and duration < 1.5 )) then
				timers[slot]:SetText("")
				if condition then
					buttons[slot]:SetVertexColor(1, 1, 1, 0.75)
				else
					buttons[slot]:SetVertexColor(1, 1, 1, 0.5)
				end
			else
				-- on cooldown
				expire = start + duration - GetTime()
				if shorttimer == 1 then 
					timers[slot]:SetText(Boomkinator:FormatShort(expire))
				else
					timers[slot]:SetText(Boomkinator:FormatTime(expire))
				end
				buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
			end
		else
			timers[slot]:SetText("NA")
			buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
		end
	-- rechargeable spell
	-- 7, x, spell, duration(1 for short)		
	elseif thing == 7 then
		spell = select(1, ...)
		shorttimer = select(2, ...)
		spellName = GetSpellInfo(spell)
		buffCount, _, _, _ = GetSpellCharges(spellName)
		if buffCount then
			if  buffCount > 0 then
				stacks[slot]:SetText("x" .. buffCount)
				timers[slot]:SetText("")
				if buffCount > 2 then
					buttons[slot]:SetVertexColor(1, 1, 1, 1)
				elseif buffCount == 2 then
					buttons[slot]:SetVertexColor(1, 1, 1, 0.75)			
				elseif buffCount == 1 then			
					buttons[slot]:SetVertexColor(1, 1, 1, 0.50)
				end
			else
				stacks[slot]:SetText("")
				buttons[slot]:SetVertexColor(1, 1, 1, 0.3)
				--check cooldown for next
				start, duration = GetSpellCooldown(spellName)
				if start then
					if start == 0 or (start > 0 and duration < 1.5 ) then
						timers[slot]:SetText("")
					else
						expire = start + duration - GetTime()
						if shorttimer == 1 then
							timers[slot]:SetText(Boomkinator:FormatShort(expire))
						else
							timers[slot]:SetText(Boomkinator:FormatTime(expire))
						end
					end
				end
			end -- buff count
		end	-- buff exists
	end -- if thing
end -- do the thing
-------------------------------------------------------------------
-- Buff UI Setup
-------------------------------------------------------------------
function Boomkinator:UpdateLayout()
	--self:Print("Update Layout")
	local spellName, spellIcon, spellSelected, spellID, i
	
	self:CheckTalents()
	icons = {}
	if updater then	-- set default icons for 10 buttons
		for i = 1, 10, 1 do
			icons[i] = select(3, GetSpellInfo(spells[i]))
		end
	end
	--self:Print(updater)
	-- talent customizations
	if updater == "ButtonsBalance" then
		-- incarnation check
		spellSelected = select(4, GetTalentInfo(5, 2, 1))
		if spellSelected then
			-- flare disabled
			spells[2] = nil
			icons[2] = nil
			-- alignment replaced by incarnation
			spells[8] = spells[11]
			icons[8] = select(3, GetSpellInfo(spells[8]))
		end
	elseif updater == "ButtonsOutlaw" then
		-- slice and dice
		slice = select(4, GetTalentInfo(7, 1, 1))
		if slice then 
			spells[7] = spells[17]
			icons[7] = select(3, GetSpellInfo(spells[7]))
		else -- bones
			spells[7] = spells[18]
			icons[7] = select(3, GetSpellInfo(spells[7]))
			for i = 11, 16, 1 do	-- bone icons
				icons[i] = select(3, GetSpellInfo(spells[i]))
			end
		end
	elseif updater == "ButtonsAffliction" then
		-- haunt check
		spellSelected = select(4, GetTalentInfo(1, 1, 1))
		if not spellSelected then
			spells[6] = nil
			icons[6] = nil
		end
	elseif updater == "ButtonsDemonology" then
		-- implosion
		implode = select(4, GetTalentInfo(2, 3, 1))
		if not implode then
			icons[7] = nil
		end
	elseif updater == "ButtonsDestruction" then
		-- roaring blaze
		blaze = select(4, GetTalentInfo(1, 2, 1))
		-- shadow burn
		burn = select(4, GetTalentInfo(1, 3, 1))
		if not burn then
			icons[2] = nil
		end
		-- wreak havoc
		wreak = select(4, GetTalentInfo(7, 1, 1))
	elseif updater == "ButtonsBeastMastery" then
		--90 talent
		spellSelected = select(4, GetTalentInfo(6, 2, 1))
		if spellSelected then --barrage
			spells[3] = spells[11] 
			icons[3] = select(3, GetSpellInfo(spells[3]))
		end
		spellSelected = select(4, GetTalentInfo(6, 3, 1))
		if spellSelected then -- volley, not tracking
			spells[3] = nil
			icons[3] = nil
		end
		--100 talent
		spellSelected = select(4, GetTalentInfo(7, 1, 1))
		if not spellSelected then -- nothing to track
			spells[9] = nil
			icons[0] = nil
		end
	end
	if updater then
		bframe = _G["BoomkinatorFrame"]
		for i = 1, 10 do
			buttons[i] = _G["BoomkinatorFrameButton"..i]
			buttons[i]:SetTexture(icons[i])
			buttons[i]:SetVertexColor(1, 1, 1, 1)
			timers[i]  = _G["BoomkinatorFrameTime"..i]
			timers[i]:SetText("")
			timers[i]:SetFont("Fonts\\FRIZQT__.TTF", settings.fontscale)
			stacks[i]  = _G["BoomkinatorFrameStack"..i]
			stacks[i]:SetText("")
			stacks[i]:SetFont("Fonts\\FRIZQT__.TTF", settings.fontscale)
		end
	end
	if settings.rows == 1 then
		buttons[2]:SetPoint("TOPLEFT", buttons[1], "TOPRIGHT", 2, 0)
		buttons[3]:SetPoint("TOPLEFT", buttons[2], "TOPRIGHT", 2, 0)
		buttons[4]:SetPoint("TOPLEFT", buttons[3], "TOPRIGHT", 2, 0)
		buttons[5]:SetPoint("TOPLEFT", buttons[4], "TOPRIGHT", 2, 0)
		buttons[6]:SetPoint("TOPLEFT", buttons[5], "TOPRIGHT", 2, 0)
		buttons[7]:SetPoint("TOPLEFT", buttons[6], "TOPRIGHT", 2, 0)
		buttons[8]:SetPoint("TOPLEFT", buttons[7], "TOPRIGHT", 2, 0)
		buttons[9]:SetPoint("TOPLEFT", buttons[8], "TOPRIGHT", 2, 0)
		buttons[10]:SetPoint("TOPLEFT", buttons[9], "TOPRIGHT", 2, 0)
	elseif settings.rows == 2 then
		buttons[2]:SetPoint("TOPLEFT", buttons[1], "TOPRIGHT", 2, 0)
		buttons[3]:SetPoint("TOPLEFT", buttons[2], "TOPRIGHT", 2, 0)
		buttons[4]:SetPoint("TOPLEFT", buttons[3], "TOPRIGHT", 2, 0)
		buttons[5]:SetPoint("TOPLEFT", buttons[4], "TOPRIGHT", 2, 0)
		buttons[6]:SetPoint("TOPLEFT", buttons[1], "BOTTOMLEFT", 0, -1)
		buttons[7]:SetPoint("TOPLEFT", buttons[6], "TOPRIGHT", 2, 0)
		buttons[8]:SetPoint("TOPLEFT", buttons[7], "TOPRIGHT", 2, 0)
		buttons[9]:SetPoint("TOPLEFT", buttons[8], "TOPRIGHT", 2, 0)
		buttons[10]:SetPoint("TOPLEFT", buttons[9], "TOPRIGHT", 2, 0)
	else
		buttons[2]:SetPoint("TOPLEFT", buttons[1], "TOPRIGHT", 2, 0)
		buttons[3]:SetPoint("TOPLEFT", buttons[2], "TOPRIGHT", 2, 0)
		buttons[4]:SetPoint("TOPLEFT", buttons[1], "BOTTOMLEFT", 0, -1)
		buttons[5]:SetPoint("TOPLEFT", buttons[4], "TOPRIGHT", 2, 0)
		buttons[6]:SetPoint("TOPLEFT", buttons[5], "TOPRIGHT", 2, 0)
		buttons[7]:SetPoint("TOPLEFT", buttons[4], "BOTTOMLEFT", 0, -1)
		buttons[8]:SetPoint("TOPLEFT", buttons[7], "TOPRIGHT", 2, 0)
		buttons[9]:SetPoint("TOPLEFT", buttons[8], "TOPRIGHT", 2, 0)
		buttons[10]:SetPoint("TOPLEFT", buttons[9], "TOPRIGHT", 2, 0)
	end
	
	BoomkinatorFrame:SetScale(settings.buffscale)
	self:HideLayout()

	if settings.locked then
		Boomkinator:LockLayout()
	else
		Boomkinator:UnlockLayout()
	end
end

function Boomkinator:ShowLayout()
	bframe:Show()
end

function Boomkinator:HideLayout()
	bframe:Hide()
end

function Boomkinator:LockLayout()
	bframe:EnableMouse(false)
end

function Boomkinator:UnlockLayout()
	bframe:EnableMouse(true)
end

function Boomkinator:GetSeverityColor(percent)
	if (percent >= 0.5) then
		return (1.0-percent)*2, 1.0, 0.0
	else
		return 1.0, percent*2, 0.0
	end
end
-------------------------------------------------------------------
-- Class Functionality: Warlock Affliction
-------------------------------------------------------------------
function Boomkinator:ButtonsAffliction()
	-- soul shards
	Boomkinator:DoTheThing(1, 5, UnitPower("player", 7))
	--check agony
	Boomkinator:DoTheThing(3, 1, "target", spells[1], 1)
	--check corruption
	Boomkinator:DoTheThing(3, 2, "target", spells[2], 1)
	-- siphon
	Boomkinator:DoTheThing(3, 3, "target", spells[3], 1)
	-- check unstable affliction	
	Boomkinator:DoTheThing(3, 5, "target", spells[5], 1)
	--check haunt
	Boomkinator:DoTheThing(3, 6, "target", spells[6], 1)		
	-- doomguard
	Boomkinator:DoTheThing(4, 7, spells[7])	
	-- grimoire
	Boomkinator:DoTheThing(4, 8, spells[8])	
	-- artifact
	local spellName = GetSpellInfo(spells[9])
	local buffCount = select(4, UnitBuff("player", spellName))
	-- collected souls
	Boomkinator:DoTheThing(1, 9, buffCount)
	-- timer
	Boomkinator:DoTheThing(2, 9, "player", spells[10], -1, -1)
end
-------------------------------------------------------------------
-- Class Functionality: Warlock Demonology
-------------------------------------------------------------------
function Boomkinator:ButtonsDemonology()
    local power = UnitPower("player" , 7)  -- SPELL_POWER_SOUL_SHARDS
	local spellName, spellIcon, buffName
	-- soul shards
	Boomkinator:DoTheThing(1, 5, power)
	if power >= 3 then
		buttons[5]:SetVertexColor(1, 1, 1, 0.75)
	elseif power >= 2 then
		buttons[5]:SetVertexColor(1, 1, 1, 0.5)
	else
		buttons[5]:SetVertexColor(1, 1, 1, 0.25)	
	end
	-- check doom
	Boomkinator:DoTheThing(3, 1, "target", spells[1], 1)
	-- demon rush
	Boomkinator:DoTheThing(2, 4, "player", spells[12], -1, 1)
	-- dreadstalkers
	-- check demonic calling
	spellName = GetSpellInfo(spells[11])
	buffName, _, _, _, _, _, _ = UnitBuff("player", spellName)
	Boomkinator:DoTheThing(6, 2, spells[2], (power >= 2 or buffName))
	-- artifact
	Boomkinator:DoTheThing(4, 8, spells[8])
	-- custom pet actions
	spellName = GetSpellInfo(spells[3])
	spellIcon = select(3, GetSpellInfo(spells[3]))
	buttons[3]:SetTexture(spellIcon)
	-- pet cooldown
	Boomkinator:DoTheThing(4, 3, spells[3])
	
	-- implosion, demonic empowerment and consumption
	if zpetsplayerguid then -- zpets installed and active
		if zpetspetsneedde > 0 then
			stacks[6]:SetText("x"..zpetspetsneedde)
			stacks[6]:SetTextColor(1, 0.2, 0.2, 1)
			buttons[6]:SetVertexColor(1, 1, 1, 0.75)
		else
			stacks[6]:SetText("")
			buttons[6]:SetVertexColor(1, 1, 1, 0.25)
		end
		
		-- check demonic empowerment		
		Boomkinator:DoTheThing(2, 6, "pet", spells[6], 0, -1)
		
		-- pet and implosion count
		if implode then 
			stacks[7]:SetText("x" .. zpetsdreadcount + zpetstotalimpcount)
			timers[7]:SetText(Boomkinator:ReadableNumber(zpetstotalvalueofimplosion))
			if zpetstotalvalueofimplosion > 200000 then
				buttons[7]:SetVertexColor(1, 1, 1, 0.50)
				timers[7]:SetTextColor(0.4, 1, 0, 1)
			else
				buttons[7]:SetVertexColor(1, 1, 1, 0.25)
				timers[7]:SetTextColor(0.2, 1, 1, 1)
			end
		end
		-- consumption count
		stacks[8]:SetText(Boomkinator:ReadableNumber(zpetsthalkielstotalvalue))
	else
		-- check demonic empowerment		
		Boomkinator:DoTheThing(2, 6, "pet", spells[6], -1)
	end
	
	-- doomguard
	Boomkinator:DoTheThing(4, 9, spells[9])	
	-- grimoire
	Boomkinator:DoTheThing(4, 10, spells[10])	
end
-------------------------------------------------------------------
-- Class Functionality: Warlock Destruction
-------------------------------------------------------------------
function Boomkinator:ButtonsDestruction()
    local power
	local guid

	-- soul shards
	power = UnitPower("player" , 7)  -- SPELL_POWER_SOUL_SHARDS 
	Boomkinator:DoTheThing(1, 4, power)
	if power < 2 then
		buttons[4]:SetVertexColor(1, 1, 1, 0.75)
	elseif power < 4 then
		buttons[4]:SetVertexColor(1, 1, 1, 0.5)
	else
		buttons[4]:SetVertexColor(1, 1, 1, 0.3)	
	end
	--immolate
	Boomkinator:DoTheThing(3, 1, "target", spells[1], 1)
	--self:Print(blaze)
	if blaze then
		guid = UnitGUID("target")
		--self:Print(guid)
		--self:Print(immolateTable[guid])
		if immolateTable[guid] then
			--self:Print("in the table")
			stacks[1]:SetText("x"..sformat("%0.1f", immolateTable[guid]))
		else
			stacks[1]:SetText("")
		end
	end
	-- chaos bolt
	-- check eradicate
	Boomkinator:DoTheThing(3, 3, "target", spells[11], 0)
	-- has shards to use
	if power >= 2 then
		buttons[3]:SetVertexColor(1, 1, 1, 1)
	else
		buttons[3]:SetVertexColor(1, 1, 1, 0.3)
	end
	
	-- shadowburn
	if UnitHealth("target")/UnitHealthMax("target") < 0.2 and power >= 1 and burn then
		buttons[2]:SetVertexColor(1, 1, 1, 0.75)
	else
		-- hold shadowburn
		buttons[2]:SetVertexColor(1, 1, 1, 0.25)
	end
	-- conflag
	Boomkinator:DoTheThing(7, 5, spells[5], 1)
	-- rift
	Boomkinator:DoTheThing(7, 9, spells[9], 1)
	-- Havoc
	if not wreak then -- on cooldown
		Boomkinator:DoTheThing(4, 6, spells[6])
	else
		Boomkinator:DoTheThing(3, 6, "focus", spells[6], 1)
	end
	-- doomguard
	Boomkinator:DoTheThing(4, 7, spells[7])	
	-- grimoire
	Boomkinator:DoTheThing(4, 8, spells[8])	
	
end
-------------------------------------------------------------------
-- Class Functionality: Rogue Combat
-------------------------------------------------------------------
function Boomkinator:ButtonsOutlaw()
	local power
	local dice = 0
	local tb = 0
	local condition = false
	-- combo points
	power = UnitPower("player", 4)
	Boomkinator:DoTheThing(1, 7, power)

	if slice then
		Boomkinator:DoTheThing(2, 7, "player", spells[7], -1, -1)
	else
		buttons[7]:SetVertexColor(1, 1, 1, 0.5)
		-- bone analysis
		for i = 1, 6 do
			spellName = GetSpellInfo(spells[i+10])
			buffName, _, _, _, _, _, buffExpire = UnitBuff("player", spellName)
			if buffName and buffExpire then
				expire = buffExpire - GetTime()
				dice = dice + 1
				if i == 1 then tb = 1 end
				if dice == 1 then
					buttons[8]:SetTexture(icons[i+10])
					buttons[8]:SetVertexColor(1, 1, 1, 0.5)
					timers[8]:SetText(Boomkinator:FormatTime(expire))
				end
			end
		end
		if dice == 0 then
			buttons[8]:SetTexture(nil)
			timers[8]:SetText("")
		else
			if dice > 1 then
				stacks[8]:SetText("+" .. ( dice - 1) )
				stacks[8]:SetTextColor(0, 1, 1, 1)
			else
				stacks[8]:SetText("")
			end
		end
	end
		
	-- kick
	Boomkinator:DoTheThing(4, 1, spells[1], 1)
	-- blade flurry
	Boomkinator:DoTheThing(2, 2, "player", spells[2], -1, -1)
	-- adrenaline rush
	--if (tb == 1 or slice) then condition = true end
	Boomkinator:DoTheThing(6, 3, spells[3], (tb == 1 or slice)) --condition)
	-- opportiunity
	Boomkinator:DoTheThing(2, 4, "player", spells[4], 1, -1)
	-- vial
	Boomkinator:DoTheThing(4, 5, spells[5])
	-- shot
	Boomkinator:DoTheThing(4, 6, spells[6])
	-- blades
	Boomkinator:DoTheThing(4, 9, spells[9])
end
-------------------------------------------------------------------
-- Class Functionality: Druid Moonkin
-------------------------------------------------------------------
function Boomkinator:ButtonsBalance()
	local power = UnitPower("player", SPELL_POWER_ECLIPSE)
	Boomkinator:DoTheThing(1, 5, power)	
	-- starsurge
	if power >= 40 then
		buttons[5]:SetVertexColor(1, 1, 1, 0.75)
	else
		buttons[5]:SetVertexColor(1, 1, 1, 0.25)
	end
	-- starfall
	if power >= 60 then
		buttons[7]:SetVertexColor(1, 1, 1, 0.75)
	else
		buttons[7]:SetVertexColor(1, 1, 1, 0.25)
	end
	-- lunar strike
	Boomkinator:DoTheThing(2, 4, "player", spells[12], 0, 1, 3)
	buttons[4]:SetVertexColor(1, 1, 1, 0.3)
	-- solar wrath
	Boomkinator:DoTheThing(2, 6, "player", spells[13], 0, 1, 3)
	buttons[6]:SetVertexColor(1, 1, 1, 0.3)
	-- artifact dynamic icon
	spellName = GetSpellInfo(spells[9])
	spellIcon = GetSpellTexture(spellName)
	buttons[9]:SetTexture(spellIcon)
	-- artifact charges
	Boomkinator:DoTheThing(7, 9, spells[9], -1)
	-- moonfire
	Boomkinator:DoTheThing(3, 1, "target", spells[1], 1)
	-- stellar flare
	Boomkinator:DoTheThing(5, 2, "target", spells[2], 1, (power >= 15))
	-- sun fire
	Boomkinator:DoTheThing(3, 3, "target", spells[3], 1)
	-- alignment
	Boomkinator:DoTheThing(4, 8, spells[8])
end
-- resto
function Boomkinator:ButtonsRestoration()
	Boomkinator:DoTheThing(2, 1, "target", spells[1], -1)
	Boomkinator:DoTheThing(2, 2, "target", spells[2], -1)
	Boomkinator:DoTheThing(2, 3, "target", spells[3], -1)
	Boomkinator:DoTheThing(7, 5, spells[5], 1)
	Boomkinator:DoTheThing(4, 6, spells[6], 1)
	Boomkinator:DoTheThing(4, 7, spells[7], -1)
	Boomkinator:DoTheThing(4, 8, spells[8], -1)
	Boomkinator:DoTheThing(4, 9, spells[9], -1)
end
-------------------------------------------------------------------
-- Class Functionality: Demon Hunter Havoc
-------------------------------------------------------------------
function Boomkinator:ButtonsHavoc()
	-- fury
	local power = UnitPower("player", SPELL_POWER_DEMONIC_FURY)
	Boomkinator:DoTheThing(1, 4, power)
	-- consume magic
	Boomkinator:DoTheThing(4, 1, spells[1], 1)
	-- fel rush
	Boomkinator:DoTheThing(7, 2, spells[2], 1)
	-- eye beam
	Boomkinator:DoTheThing(6, 3, spells[3], (power >= 50))
	-- chaos strike
	Boomkinator:DoTheThing(6, 4, spells[4], (power >= 40), 1)
	-- throw glaive
	Boomkinator:DoTheThing(4, 5, spells[5], 1)
	-- blade dance
	Boomkinator:DoTheThing(6, 6, spells[6], (power >= 35), 1)
	-- meta
	Boomkinator:DoTheThing(4, 7, spells[7])
	-- artifact
	Boomkinator:DoTheThing(4, 8, spells[8])
	-- retreat
	Boomkinator:DoTheThing(4, 9, spells[9])
end
-------------------------------------------------------------------
-- Class Functionality: Demon Hunter Vengeance
-------------------------------------------------------------------
function Boomkinator:ButtonsVengeance()
	local power = UnitPower("player", SPELL_POWER_PAIN)
	Boomkinator:DoTheThing(1, 4, power)
	buttons[4]:SetVertexColor(1, 1, 1, 0.25)
	-- consume magic
	Boomkinator:DoTheThing(4, 1, spells[1], 1)
	-- infernal strike
	Boomkinator:DoTheThing(7, 2, spells[2], 1)
	-- sigil of flame
	Boomkinator:DoTheThing(4, 3, spells[3], 1)
	-- soul cleave
	Boomkinator:DoTheThing(2, 5, "player", spells[11], 0, 1)
	if power >= 60 then
		buttons[5]:SetVertexColor(1, 1, 1, 0.75)
	elseif power >= 30 then
		buttons[5]:SetVertexColor(1, 1, 1, 0.50)
	else
		buttons[5]:SetVertexColor(1, 1, 1, 0.25)
	end
	-- immolation aura
	Boomkinator:DoTheThing(4, 6, spells[6], 1)
	-- meta
	Boomkinator:DoTheThing(4, 7, spells[7])
	-- demon spikes
	Boomkinator:DoTheThing(7, 8, spells[8], 1)
	-- empower wards
	Boomkinator:DoTheThing(4, 9, spells[9], 1)
	-- artifact
	Boomkinator:DoTheThing(4, 10, spells[10])
end
-------------------------------------------------------------------
-- Class Functionality: Hunter Beast Mastery
-------------------------------------------------------------------
function Boomkinator:ButtonsBeastMastery()
	local power = UnitPower("player", SPELL_POWER_FOCUS)
	Boomkinator:DoTheThing(1, 4, power)
	if power >= 90 then
		buttons[4]:SetVertexColor(1, 1, 1, 0.75)
	elseif power >= 40 then
		buttons[4]:SetVertexColor(1, 1, 1, 0.5)
	else
		buttons[4]:SetVertexColor(1, 1, 1, 0.25)
	end
	--counter shot
	Boomkinator:DoTheThing(4, 1, spells[1], 1)
	-- beast cleave
	Boomkinator:DoTheThing(2, 2, "pet", spells[2], -1, -1, 0)
	-- murder
	Boomkinator:DoTheThing(6, 3, spells[3], (power >= 30), -1)
	-- kill command
	Boomkinator:DoTheThing(6, 5, spells[5], (power >= 30), 1)
	-- dire beast
	Boomkinator:DoTheThing(4, 6, spells[6], 1)
	-- bestial wrath
	Boomkinator:DoTheThing(4, 7, spells[7], -1)
	-- aspect of the wild
	Boomkinator:DoTheThing(4, 8, spells[8], -1)
	-- stampede
	Boomkinator:DoTheThing(4, 9, spells[9], -1)
end
-------------------------------------------------------------------
-- Class Functionality: Hunter Beast Mastery
-------------------------------------------------------------------
function Boomkinator:ButtonsRetribution()
	local power = UnitPower("player", SPELL_POWER_HOLY_POWER)
	Boomkinator:DoTheThing(1, 7, power)
	Boomkinator:DoTheThing(4, 1, spells[1], 1)
	Boomkinator:DoTheThing(6, 2, spells[2], (power >= 3), 1)
	Boomkinator:DoTheThing(6, 3, spells[3], (power == 5), 1)
	Boomkinator:DoTheThing(7, 4, spells[4], 1)
	Boomkinator:DoTheThing(4, 5, spells[5], 1)
	Boomkinator:DoTheThing(4, 6, spells[6], 1)
	Boomkinator:DoTheThing(6, 7, spells[7], (power >= 3), 1)
	Boomkinator:DoTheThing(4, 8, spells[8], -1)
	
end